C+    hopf_part.f
C This file was written by Jonathan A. Sherratt
C Copyright (C) 2013 Heriot-Watt University, Edinburgh, UK
C  
C THIS FILE IS PART OF WAVETRAIN.
C WAVETRAIN SOFTWARE IS INTENDED FOR FREE PERSONAL PRIVATE 
C AND ACADEMIC USE ONLY AND WAS DEVELOPED BY JONATHAN SHERRATT AT 
C HERIOT-WATT UNIVERSITY, EDINBURGH, UK, AND IS PROTECTED BY COPYRIGHT. 
C THE COPYRIGHT OWNER IS HERIOT-WATT UNIVERSITY, WITH THE EXCEPTION OF 
C THE ROUTINES FROM THE PACKAGES AUTO, BLAS, EISPACK, LAPACK AND MINPACK, 
C WHICH ARE INCLUDED WITH PERMISSION, WHICH ARE CLEARLY IDENTIFIED IN 
C THE SOURCE CODE, AND THE USE OF THESE PACKAGES IS UNDER THE CONDITIONS 
C OF THE ORIGINAL OWNERS WHICH ARE AVAILABLE AT 
C http://www.netlib.org/blas/faq.html#2;
C http://www.netlib.org/lapack/#_licensing; 
C http://www.netlib.org/minpack/disclaimer.
C 
C PERSONS OR ORGANISATIONS WISHING TO USE THIS WAVETRAIN SOFTWARE FOR 
C COMMERCIAL PURPOSES OR PURPOSES OTHER THAN ITS ORIGINAL INTENTION 
C SHOULD CONTACT THE OWNERS TO REQUEST SPECIFIC WRITTEN PERMISSIONS.
C 
C DISTRIBUTION: DISTRIBUTION OF WAVETRAIN SOFTWARE, IN PART OR AS 
C A WHOLE, BY ANY MEANS (ELECTRONIC, PRINTED COPY OR OTHERWISE), IS 
C PROHIBITED WITHOUT THE PRIOR WRITTEN PERMISSION OF EITHER HERIOT-WATT 
C UNIVERSITY OR THE AUTHOR JONATHAN A. SHERRATT. ALL REQUESTS FOR SUCH 
C PERMISSION SHOULD BE SENT BY EMAIL TO J.A.SHERRATT@HW.AC.UK; PLEASE 
C INCLUDE THE WORD WAVETRAIN IN THE SUBJECT HEADING OF THE MESSAGE.  
C 
C WARRANTY LIMITATIONS: THE PRODUCT AND SERVICE ARE PROVIDED "AS IS". 
C THE OWNER DISCLAIMS ANY AND ALL WARRANTIES, INCLUDING BUT NOT LIMITED 
C TO, ALL EXPRESSED OR IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
C FOR A PARTICULAR PURPOSE. UNDER NO CIRCUMSTANCES WILL THE OWNER BE 
C LIABLE TO YOU FOR ANY LOSS OF USE, INTERRUPTION OF BUSINESS OR ANY 
C DIRECT, INDIRECT, SPECIAL, INCIDENTAL, AND/OR CONSEQUENTIAL DAMAGES 
C OF ANY KIND (INCLUDING LOSS OF PROFITS) EVEN IF THE OWNER HAS BEEN 
C ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. THE OWNER ACCEPTS NO LEGAL 
C LIABILITY FOR ANY OF THE PRODUCT AND/OR SERVICE SOLD, OFFERED FOR SALE, 
C OR OTHERWISE MADE AVAILABLE FOR PUBLIC OR PRIVATE CONSUMPTION IN 
C RELATION TO THIS SOFTWARE. THE OWNER DOES NOT WARRANT THAT THIS 
C SOFTWARE WILL BE ERROR-FREE.
C  
C+    THIS PROGRAM IS COMBINED WITH equations.f TO FORM THE MAIN
C+    AUTO PROGRAM FOR CALCULATIONS OF STABILITY BOUNDARIES OF
C+    HOPF TYPE.
C++
C----------------------------------------------------------------------
      SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION U(NDIM), PAR(*), F(NDIM), ICP(*)
      DIMENSION UPTW(NDIM/5),DUPTWZ(NDIM/5)
      DIMENSION AMATRX(NDIM/5,NDIM/5)
      DIMENSION BMATRX(NDIM/5,NDIM/5)
      DIMENSION UEFNR(NDIM/5),UEFNI(NDIM/5)
      DIMENSION UEFNGR(NDIM/5),UEFNGI(NDIM/5)
C-----
      NPTW=NDIM/5
      PP=PAR(1)
      C=PAR(2)
      PERIOD=PAR(3)
      GAMMA=PAR(4)
      EVALR=PAR(5)
C     Re(e-value)
      EVALI=PAR(6)
C     Im(e-value)
      EVALGR=PAR(7)
C     Real part of the first derivative of the e-value wrt i*gamma
      EVALGI=PAR(8)
C     Imag part of the first derivative of the e-value wrt i*gamma
C     Note that EVALGR=(d/dgamma)(EVALI) and EVALGI=-(d/dgamma)(EVALR)
C-----
      DO 100 I=1,NPTW
         UPTW(I)=U(I)
 100  CONTINUE
      CALL TWEQS(NPTW,PP,C,UPTW,DUPTWZ)
      DO 110 I=1,NPTW
         F(I)=PERIOD*DUPTWZ(I)
 110  CONTINUE
C-----
      CALL EFEQNS(NPTW,PP,C,UPTW,AMATRX,BMATRX)
C-----
      DO 200 I=1,NPTW
         UEFNR(I)=U(2*I+NPTW-1)
         UEFNI(I)=U(2*I+NPTW)
 200  CONTINUE
C     Note: these solution arrays are the eigenfunction multiplied
C           by exp(-i*gamma*z). In the notation of Rademacher et al
C           Physica D 229: 166 (2007), section 3.4.3, these variables
C           are V (rather than U). This is different from 
C           spectrum_part.f, where variables of the same name denote 
C           U. For this reason, the boundary conditions are different 
C           in this program and in spectrum_part.f
C-----
      DO 210 I=1,NPTW
         DUEFZR=GAMMA*UEFNI(I)
         DUEFZI=-GAMMA*UEFNR(I)
C        Note: in equation (3.11) of Rademacher et al Physica D 229: 
C              166 (2007), the term corresponding to the above two
C              lines is multiplied by L. This is a mistake in their
C              paper.
         DO 220 J=1,NPTW
            DUEFZR=DUEFZR+PERIOD*(
     &           (AMATRX(I,J)+EVALR*BMATRX(I,J))*UEFNR(J)
     &           -EVALI*BMATRX(I,J)*UEFNI(J))
            DUEFZI=DUEFZI+PERIOD*(
     &           +(AMATRX(I,J)+EVALR*BMATRX(I,J))*UEFNI(J)
     &           +EVALI*BMATRX(I,J)*UEFNR(J))
 220     CONTINUE
         F(2*I+NPTW-1)=DUEFZR
         F(2*I+NPTW)=DUEFZI
 210  CONTINUE
C-----
      DO 300 I=1,NPTW
         UEFNGR(I)=U(2*I+3*NPTW-1)
         UEFNGI(I)=U(2*I+3*NPTW)
 300  CONTINUE
C-----
      DO 310 I=1,NPTW
         DEGZR=-UEFNR(I)+GAMMA*UEFNGI(I)
         DEGZI=-UEFNI(I)-GAMMA*UEFNGR(I)
C        Note: in equation (3.11) of Rademacher et al Physica D 229: 
C              166 (2007), the term corresponding to the above two
C              lines is multiplied by L. This is a mistake in their
C              paper.
         DO 320 J=1,NPTW
            DEGZR=DEGZR+PERIOD*(
     &       (AMATRX(I,J)+EVALR*BMATRX(I,J))*UEFNGR(J)
     &       -EVALI*BMATRX(I,J)*UEFNGI(J)
     &       +EVALGR*BMATRX(I,J)*UEFNR(J)
     &       -EVALGI*BMATRX(I,J)*UEFNI(J))
            DEGZI=DEGZI+PERIOD*(
     &       (AMATRX(I,J)+EVALR*BMATRX(I,J))*UEFNGI(J)
     &       +EVALI*BMATRX(I,J)*UEFNGR(J)
     &       +EVALGR*BMATRX(I,J)*UEFNI(J)
     &       +EVALGI*BMATRX(I,J)*UEFNR(J))
 320     CONTINUE
         F(2*I+3*NPTW-1)=DEGZR
         F(2*I+3*NPTW)=DEGZI
 310  CONTINUE
C-----
      RETURN
      END
C----------------------------------------------------------------------
      SUBROUTINE STPNT(NDIM,U,PAR)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION U(NDIM), PAR(*)
      OPEN(21,FILE='../ptwcalc/data_files/ptw.params'
     &,STATUS='OLD')
      REWIND(21)
C     Note that 21 is not used in auto97 as either a file number,
C     a position label or a format statement.
      READ(21,*)PP,C,PERIOD
      CLOSE(21)    
      PAR(1)=PP
      PAR(2)=C
      PAR(3)=PERIOD
      OPEN(21,FILE='../loci/data_files/hopf.gstart'
     &,STATUS='OLD')
      REWIND(21)
C     Note that 21 is not used in auto97 as either a file number,
C     a position label or a format statement.
      READ(21,*)PAR(4)
      CLOSE(21)
      OPEN(21,FILE='../temp_files/extracthopf.params'
     &,STATUS='OLD')
      REWIND(21)
C     Note that 21 is not used in auto97 as either a file number,
C     a position label or a format statement.
      READ(21,*)PAR(5)
      READ(21,*)PAR(6)
      READ(21,*)PAR(7)
      READ(21,*)PAR(8)
      CLOSE(21)
      PAR(9)=0.0D0
C     par(9) is a dummy parameter
C     NOTE: SOLUTION IS INITIALISED BY READING IN A SOLUTION 
C           FROM A FILE USING THE @fc COMMAND
      RETURN
      END
C----------------------------------------------------------------------
      SUBROUTINE BCND(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC) 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION PAR(*),ICP(*),U0(NDIM),U1(NDIM),FB(NBC)
      DO 100 I=1,NBC
         FB(I)=U0(I)-U1(I)
 100  CONTINUE
C     Note the difference in boundary conditions in this program
C          compared to the corresponding conditions in 
C          spectrum_part.f, because the variables here are
C          exp(-i*gamma*z)*efn. In the notation of Rademacher et al
C          Physica D 229: 166 (2007), section 3.4.3, the variables
C          are V in this program and U in spectrum_part.f. 
      RETURN 
      END 
C----------------------------------------------------------------------
      SUBROUTINE ICND(NDIM,PAR,ICP,NINT,U,UOLD,UDOT,UPOLD,FI,IJAC,DINT)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION U(NDIM),UOLD(NDIM),UDOT(NDIM),UPOLD(NDIM)
      DIMENSION FI(NINT),DINT(NINT,*),ICP(*),PAR(*)
      NPTW=NDIM/5
      FI(1)=0.0D0
      FI(2)=-1.0D0
      FI(3)=0.0D0
      DO 100 I=1,NPTW
         FI(1)=FI(1)+UPOLD(I)*(UOLD(I)-U(I))
         JR=NPTW+2*I-1
         JI=JR+1
         FI(2)=FI(2)+U(JR)*U(JR)+U(JI)*U(JI)
         FI(3)=FI(3)+UOLD(JR)*U(JI)-UOLD(JI)*U(JR)
 100  CONTINUE
C     NOTE: u(jr) AND u(ji) ARE THE REAL AND IMAGINARY 
C           PARTS OF THE EIGENFUNCTION CORRESPONDING TO 
C           THE iTH TRAVELLING WAVE VARIABLE, MULTIPLIED
C           BY exp(-i*gamma*z).
      FI(4)=0.0D0
      FI(5)=0.0D0
      OPEN(21,FILE='../temp_files/ptw_variables.ifactors'
     &,STATUS='OLD')
      REWIND(21)
C     Note that 21 is not used in auto97 as either a file number,
C     a position label or a format statement.
      DO 300 I=1,NPTW
         READ(21,*)IFACTR
C     this will be 0 or 1
         FI(4)=FI(4)+IFACTR*
     &        (U(2*I+NPTW-1)*U(2*I+3*NPTW-1)
     &        +U(2*I+NPTW)*U(2*I+3*NPTW))
         FI(5)=FI(5)+IFACTR*
     &        (U(2*I+NPTW)*U(2*I+3*NPTW-1)
     &        -U(2*I+NPTW-1)*U(2*I+3*NPTW))
 300  CONTINUE
      CLOSE(21)
      RETURN 
      END 
C----------------------------------------------------------------------
      SUBROUTINE FOPT 
      RETURN 
      END 
C----------------------------------------------------------------------
      SUBROUTINE PVLS
      RETURN 
      END 
C----------------------------------------------------------------------
