#+ pplane_key.gp
# This file was written by Jonathan A. Sherratt
# Copyright (C) 2013 Heriot-Watt University, Edinburgh, UK
#  
# THIS FILE IS PART OF WAVETRAIN.
# WAVETRAIN SOFTWARE IS INTENDED FOR FREE PERSONAL PRIVATE 
# AND ACADEMIC USE ONLY AND WAS DEVELOPED BY JONATHAN SHERRATT AT 
# HERIOT-WATT UNIVERSITY, EDINBURGH, UK, AND IS PROTECTED BY COPYRIGHT. 
# THE COPYRIGHT OWNER IS HERIOT-WATT UNIVERSITY, WITH THE EXCEPTION OF 
# THE ROUTINES FROM THE PACKAGES AUTO, BLAS, EISPACK, LAPACK AND MINPACK, 
# WHICH ARE INCLUDED WITH PERMISSION, WHICH ARE CLEARLY IDENTIFIED IN 
# THE SOURCE CODE, AND THE USE OF THESE PACKAGES IS UNDER THE CONDITIONS 
# OF THE ORIGINAL OWNERS WHICH ARE AVAILABLE AT 
# http://www.netlib.org/blas/faq.html#2;
# http://www.netlib.org/lapack/#_licensing; 
# http://www.netlib.org/minpack/disclaimer.
# 
# PERSONS OR ORGANISATIONS WISHING TO USE THIS WAVETRAIN SOFTWARE FOR 
# COMMERCIAL PURPOSES OR PURPOSES OTHER THAN ITS ORIGINAL INTENTION 
# SHOULD CONTACT THE OWNERS TO REQUEST SPECIFIC WRITTEN PERMISSIONS.
# 
# DISTRIBUTION: DISTRIBUTION OF WAVETRAIN SOFTWARE, IN PART OR AS 
# A WHOLE, BY ANY MEANS (ELECTRONIC, PRINTED COPY OR OTHERWISE), IS 
# PROHIBITED WITHOUT THE PRIOR WRITTEN PERMISSION OF EITHER HERIOT-WATT 
# UNIVERSITY OR THE AUTHOR JONATHAN A. SHERRATT. ALL REQUESTS FOR SUCH 
# PERMISSION SHOULD BE SENT BY EMAIL TO J.A.SHERRATT@HW.AC.UK; PLEASE 
# INCLUDE THE WORD WAVETRAIN IN THE SUBJECT HEADING OF THE MESSAGE.  
# 
# WARRANTY LIMITATIONS: THE PRODUCT AND SERVICE ARE PROVIDED "AS IS". 
# THE OWNER DISCLAIMS ANY AND ALL WARRANTIES, INCLUDING BUT NOT LIMITED 
# TO, ALL EXPRESSED OR IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE. UNDER NO CIRCUMSTANCES WILL THE OWNER BE 
# LIABLE TO YOU FOR ANY LOSS OF USE, INTERRUPTION OF BUSINESS OR ANY 
# DIRECT, INDIRECT, SPECIAL, INCIDENTAL, AND/OR CONSEQUENTIAL DAMAGES 
# OF ANY KIND (INCLUDING LOSS OF PROFITS) EVEN IF THE OWNER HAS BEEN 
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. THE OWNER ACCEPTS NO LEGAL 
# LIABILITY FOR ANY OF THE PRODUCT AND/OR SERVICE SOLD, OFFERED FOR SALE, 
# OR OTHERWISE MADE AVAILABLE FOR PUBLIC OR PRIVATE CONSUMPTION IN 
# RELATION TO THIS SOFTWARE. THE OWNER DOES NOT WARRANT THAT THIS 
# SOFTWARE WILL BE ERROR-FREE.
#  
#+ THIS gnuplot COMMAND FILE GENERATES A KEY TO INDICATE THE 
#+ MEANING OF THE VARIOUS COLOURS/SYMBOLS/LINE STYLES IN PLOTS
#+ OF THE CONTROL PARAMETER-WAVE SPEED PLANE. NOTE THAT THE KEY 
#+ ALWAYS SHOWS SYMBOLS CORRESPONDING TO THE OUTCOME OF THE
#+ CALCULATIONS. THE PLOTTING COMMAND @pplane ALSO ENABLES PLOTS
#+ USING WAVE PERIODS OR RCODE VALUES, BUT IT IS ASSUMED THAT 
#+ THESE WILL ONLY BE BE USED TO ASSIST IN DOING OTHER PLOTS 
#+ AND IN DETERMINING  ADDITIONAL CALCULATIONS, RATHER THAN FOR 
#+ PRESENTATIONAL PURPOSES, MAKING IT UNNECESSARY TO PROVIDE 
#+ SPECIFIC KEYS FOR THESE OTHER CASES.
#+ THERE IS ONE OPTIONAL ARGUMENT, WHICH IS A SCALING FACTOR 
#+ APPLIED TO THE WIDTH OF THE BOX AROUND THE KEY.
#++
if ("$#">1) print "ERROR: either 0 or 1 arguments are expected." ; \
            print "       If there is an argument, it is a rescaling" ; \
            print "       factor to be applied to the width of key box" ; exit
if ("$#"==0) widthscaling=1.0 ; else widthscaling="$0"
if ("$#"==0) warningtext="scaling" ; else warningtext="smaller scaling"
unset label
unset xlabel
unset ylabel
unset multiplot ; mpon="no"
unset xzeroaxis
unset yzeroaxis
set tics nooffset
nkey=words(pplanekeylist)
set size 0.9,0.9
set origin 0.05,0.05
xrangemax=int(100.0/widthscaling)
set xrange [0:xrangemax]
set yrange [0:100]
ncharsmax=0 # max number of characters in the text label associated with a key item
ikey=1 # index for item number in the list of key items
keylinecmd="" #initialisation
y=100 # initialisation
check4and5=0 # initialisation
xpoint=pplanekeygaplength+0.5*pplanekeylinelength
xtext=2*pplanekeygaplength+pplanekeylinelength
xline1=pplanekeygaplength
xline2=pplanekeygaplength+pplanekeylinelength
xpoint=xpoint/widthscaling
xtext1=xtext # xtext1 is used to calculate xmax
xtext=xtext/widthscaling
xline1=xline1/widthscaling
xline2=xline2/widthscaling
ifirst=0
load "pplane_key_nextitem.gp"
characterwidth=100.0/(0.9*canvaswidthchars) 
# This is an estimate of the average width of one character, in graph
#   coordinates, for the basic font selected when the terminal is
#   specified. The 0.9 comes from the size specification. Note that 100.0
#   rather than xrangemax is used deliberately, so that characterwidth
#   is not affected by a rescaling factor supplied as an argument.
xmax=xtext1+ncharsmax*pplanekeytextfactor*characterwidth+pplanekeygaplength
if (xmax>xrangemax) xmax=xrangemax ; \
 print "WARNING: The box that Wavetrain would like to draw around the key"; \
 print "         is too wide for the plotting region. Therefore its width" ; \
 print "         is being reduced. As a result the box may be too narrow" ; \
 print "         for the key. If this is the case, the simplest remedy is" ; \
 print "         to rerun the command with a ".warningtext." factor as an" ; \
 print "         argument. An alternative solution is to reduce the value" ; \
 print "         of pplanekeytextexpand, set in plot_defaults.input."
yboxmin=y-0.5*pplanekeylineseparation
yboxmax=100-0.5*pplanekeylineseparation
system "create_box_data.script data_files/keybox.data 0.0 ".gprintf("%18.10g",yboxmin)." ".gprintf("%18.10g",xmax)." ".gprintf("%18.10g",yboxmax)
unset xlabel
unset ylabel
unset tics
unset border
plot "data_files/keybox.data" using 1:2 with lines @linethickness linetype -1 \
 linecolor rgb "black"@keylinecmd
set border 4095 front @linethickness linetype -1 linecolor rgb "black"
# the above command restores the border setting to that specified in plot.script
set tics
