#+ spectrum_multicolour.gp
# This file was written by Jonathan A. Sherratt
# Copyright (C) 2013 Heriot-Watt University, Edinburgh, UK
#  
# THIS FILE IS PART OF WAVETRAIN.
# WAVETRAIN SOFTWARE IS INTENDED FOR FREE PERSONAL PRIVATE 
# AND ACADEMIC USE ONLY AND WAS DEVELOPED BY JONATHAN SHERRATT AT 
# HERIOT-WATT UNIVERSITY, EDINBURGH, UK, AND IS PROTECTED BY COPYRIGHT. 
# THE COPYRIGHT OWNER IS HERIOT-WATT UNIVERSITY, WITH THE EXCEPTION OF 
# THE ROUTINES FROM THE PACKAGES AUTO, BLAS, EISPACK, LAPACK AND MINPACK, 
# WHICH ARE INCLUDED WITH PERMISSION, WHICH ARE CLEARLY IDENTIFIED IN 
# THE SOURCE CODE, AND THE USE OF THESE PACKAGES IS UNDER THE CONDITIONS 
# OF THE ORIGINAL OWNERS WHICH ARE AVAILABLE AT 
# http://www.netlib.org/blas/faq.html#2;
# http://www.netlib.org/lapack/#_licensing; 
# http://www.netlib.org/minpack/disclaimer.
# 
# PERSONS OR ORGANISATIONS WISHING TO USE THIS WAVETRAIN SOFTWARE FOR 
# COMMERCIAL PURPOSES OR PURPOSES OTHER THAN ITS ORIGINAL INTENTION 
# SHOULD CONTACT THE OWNERS TO REQUEST SPECIFIC WRITTEN PERMISSIONS.
# 
# DISTRIBUTION: DISTRIBUTION OF WAVETRAIN SOFTWARE, IN PART OR AS 
# A WHOLE, BY ANY MEANS (ELECTRONIC, PRINTED COPY OR OTHERWISE), IS 
# PROHIBITED WITHOUT THE PRIOR WRITTEN PERMISSION OF EITHER HERIOT-WATT 
# UNIVERSITY OR THE AUTHOR JONATHAN A. SHERRATT. ALL REQUESTS FOR SUCH 
# PERMISSION SHOULD BE SENT BY EMAIL TO J.A.SHERRATT@HW.AC.UK; PLEASE 
# INCLUDE THE WORD WAVETRAIN IN THE SUBJECT HEADING OF THE MESSAGE.  
# 
# WARRANTY LIMITATIONS: THE PRODUCT AND SERVICE ARE PROVIDED "AS IS". 
# THE OWNER DISCLAIMS ANY AND ALL WARRANTIES, INCLUDING BUT NOT LIMITED 
# TO, ALL EXPRESSED OR IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE. UNDER NO CIRCUMSTANCES WILL THE OWNER BE 
# LIABLE TO YOU FOR ANY LOSS OF USE, INTERRUPTION OF BUSINESS OR ANY 
# DIRECT, INDIRECT, SPECIAL, INCIDENTAL, AND/OR CONSEQUENTIAL DAMAGES 
# OF ANY KIND (INCLUDING LOSS OF PROFITS) EVEN IF THE OWNER HAS BEEN 
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. THE OWNER ACCEPTS NO LEGAL 
# LIABILITY FOR ANY OF THE PRODUCT AND/OR SERVICE SOLD, OFFERED FOR SALE, 
# OR OTHERWISE MADE AVAILABLE FOR PUBLIC OR PRIVATE CONSUMPTION IN 
# RELATION TO THIS SOFTWARE. THE OWNER DOES NOT WARRANT THAT THIS 
# SOFTWARE WILL BE ERROR-FREE.
#  
#+ THIS gnuplot COMMAND FILE PLOTS THE SPECTRUM FOR THE CASE
#+ OF VARIABLE COLOURING (USING EITHER RAINBOW OR GREYSCALE
#+ COLOUR MAP) BASED ON THE VALUE OF THE PHASE DIFFERENCE
#+ ACROSS ONE PERIOD OF THE WAVE. IT IS CALLED BY
#+ THE COMMAND FILE spectrum.gp. NOTE THAT THE PLOT THAT IS
#+ GENERATED IS REALLY A 3D PLOT, SEEN "END ON"; ROTATION
#+ VIA THE MOUSE IS DISABLED BECAUSE THE PLOT IS
#+ GENERATED IN THE multiplot ENVIRONMENT.
#++
unset multiplot ; mpon="no"
system "spectrum_rescale.script"
if (system("cat ../temp_files/spectrum_rescale.error") eq "1") exit
foldposition=vrmax.",".vimax.",".vimax
if (showspectrumfold eq "yes") \
set label vrmaxlabeltext at @foldposition right front textcolor rgb "black" \
 point pointsize foldsymbolsizesp pointtype 2 linecolor rgb "black" \
 offset character -1,0
xyrescale=system("getline.script 1 ../temp_files/spectrum_rescale.output")
if (system("cat ../temp_files/getline.error") eq "1") exit
set zrange [ymin:ymax] noreverse writeback
set cbrange [0:6.2831853]
if (empty1==0 || showperiodiceigenvalues ne "yes") evg0cmd1="" ; \
 else evg0cmd1=', "data_files/evg0.dat1" using 1:2:2 with points pointtype 7 \
 pointsize periodiceigenvaluessize linecolor rgb "black"'
if (empty2==0 || showperiodiceigenvalues ne "yes") evg0cmd2="" ; \
 else evg0cmd2=', "data_files/evg0.dat2" using 1:2:2 with points pointtype 4 \
 pointsize periodiceigenvaluessize linecolor rgb "black"'
# Now set up plotting of spectrum
system "selectdata12.script data_files/spectrum.plotdata data_files/spectrum.gpdata"
if (system("cat ../temp_files/selectdata.error") eq "1") exit
set xtics offset character 0,-1.1
# IMPORTANT NOTE: the above line generates the tic labels but the tic marks
#                 are not visible because they lie in the x-y plane of the
#                 3-D plot. There does not seem to be any way around this.
unset ytics
set ztics nooffset
set xyplane 0
set xlabel xlabeltext textcolor rgb "black" offset character 0,-2.8
set ylabel
set label ylabeltext textcolor rgb "black" at xmin,(ymin+ymax)/2,(ymin+ymax)/2 center rotate by 90 offset character (-labelsfactor*1.0-6.5),0
# Note: Prior to rotation, label is right-justified against axis. 
#       The rotation is applied about the middle of the axis label.
# Set size and origin before setting multiplot so that the
#   bounding box includes all of the panels in a postscript file
set size 1.0-leftspace-rightspace,1.0-topspace-bottomspace
set origin leftspace,bottomspace+0.04*(1.0-topspace-bottomspace)
# Note that the shift in the origin by 0.04*(1.0-topspace-bottomspace) is
#  added based on trial and error to give a better bounding box
set multiplot ; mpon="yes"
if (spectrumkeyheight > 0) \
 set size 1.0-leftspace-rightspace,1.0-topspace-bottomspace-spectrumkeyheight*0.01-(spectrumkeyspacing-6.5)*0.01 ; \
 set origin leftspace,bottomspace ; \
 set style line 1001 @linethickness linetype -1 linecolor rgb "black" ; \
 unset cbtics ; \
 boxwidth=0.75*(1.0-leftspace-rightspace) ; \
 set label spectrumkeytextexpand."0".textend at screen (leftspace-rightspace)*0.5+(1-boxwidth)*0.48,1.0-topspace-spectrumkeyheight*0.01+0.005*spectrumkeyheight right offset character 0,(spectrumkeytextfactor-1.0)*0.5 ; \
 set label spectrumkeytextexpand.twopi.textend at screen (leftspace-rightspace)*0.5+boxwidth+(1-boxwidth)*0.52,1.0-topspace-spectrumkeyheight*0.01+0.005*spectrumkeyheight left offset character 0,(spectrumkeytextfactor-1.0)*0.5 ; \
 set colorbox horizontal user size screen boxwidth,spectrumkeyheight*0.01 \
 origin screen (leftspace-rightspace)*0.5+(1-boxwidth)*0.5,1.0-topspace-spectrumkeyheight*0.01 \
 border 1001 ; \
 else \
 unset colorbox ; \
 set size 1.0-leftspace-rightspace,1.0-topspace-bottomspace ; \
 set origin leftspace,bottomspace
# Note that 6.5 is subtracted from spectrumkeyspacing to give better correspondence
#   between sm and gnuplot when using xyscale=1. The actual amount that should be
#   subtracted is terminal dependent and therefore different for screen and postscript
#   output; 6.5 is a compromise.
 if (spectrumcolour eq "rainbow") \
 set palette model HSV functions gray*0.7,1,1 ; \
 else \
 set palette model HSV functions 1,0,0.1+0.85*gray 
set view 90,0,1.6*xyrescale,1.0
# Now define the main plotting command
spectrumcmd='splot "data_files/spectrum.gpdata" using 1:2:2:3 \
 with lines palette linetype -1 '.linethickness.', \
 "data_files/spectrum.xaxisdata" using 1:2:2 with lines linetype 0 linewidth 1.0 linecolor rgb "black", \
 "data_files/spectrum.yaxisdata" using 1:2:2 with lines linetype 0 linewidth 1.0 linecolor rgb "black"' 
# Finally issue the plot command.
@spectrumcmd@evg0cmd1@evg0cmd2
# End of command file.       
